/*
*	checkboxselect - Cool effect for field select on form
*	Copyright (c) 2016 Ktquez
*	Project repository: https://github.com/ktquez/checkboxselect
* 	MIT license.
*/

var checkboxselect = (function(){

	"use strict";	

	// Own configuration of each field select
	var ownConfig = {};
	// Default values
	var defaults = {
		theme : 'clean',
		search : false,
		noResults : 'No Results'
	};

	/**
	 * Utilities
	 * Included in version 1.2.0
	 */
	var _ = {
		create : function (tag){
			return document.createElement(tag);
		},
		attr : function(el, attr, value){
						if (typeof(value) == 'undefined') {
							return el.getAttribute(attr);
						}
						if (Array.isArray(el)) {
							el.forEach(function(element) {
							element.setAttribute(attr, value);	
						});
						return;
						}						
						el.setAttribute(attr, value);
		},
		events : function(el, type, callback) {
			el.addEventListener(type, callback, false);
		},
		toArray : function(el){
			return [].slice.call(el);
		},
		addClass : function(el, selector){
			var self = this;
			var cls = !!self.attr(el, 'class') ? self.attr(el, 'class') : '';
			self.attr(el, 'class',  cls+ ' '+selector);
		},
		rmClass : function(el, selector){
			var self = this;
			if (Array.isArray(el)) {
				el.forEach(function(element) {
					self.attr(element, 'class', self.attr(element, 'class').replace(' '+selector, ''));					
				});
				return;
			}
			self.attr(el, 'class', self.attr(el, 'class').replace(' '+selector, ''));
		},
		$ : function(selector, el) {
			return (el || document).querySelector(selector);
		},
		$$ : function(selector, el) {
			return this.toArray((el || document).querySelectorAll(selector));
		}
	};

	/**
	 * Starts the module preparing the elements
	 * @param config = String or object to setting
	 */
	function init(config){
		setElement(config);
		prepareElement();
	}

	/**
	 * Defines the own configuration and assigns the select
	 * @param {[type]} config = String or object to setting
	 */
	function setElement(config){

		var objConfig = typeof config === 'object' ? config : {};

		if (typeof config === 'string') {
			objConfig.el = config;
		}

		// Retrieve the DOM to be manipulated
		objConfig.DOM = _.$$(objConfig.el);

		mergeToDefaults(objConfig);		
	}


	/**
	 * Prepare the elements that will be handled by the module
	 */
	function prepareElement(){

		ownConfig.DOM.map(function(select, index){
			createElements(select, index);
		});
	}

	function createElements(select, index){

		// Cache self config 
		var config = ownConfig;

		select.style.display = 'none';

		var parent = select.parentElement;
		var grandparent = select.parentElement.parentElement;
		var placeholder = _.attr(select, 'placeholder');

		// Visual element simule field input
		var field = _.create('div');
		_.addClass(field, 'pk-slide inputcheckbox -'+ config.theme);
		if(!!placeholder) field.innerHTML = placeholder;
		if(parent.hasAttribute('for')) {
			var id=_.attr(parent, 'for')
			_.attr(field, 'id', _.attr(parent, 'for'))
		}
		else {
			if(grandparent.hasAttribute('for')) {
			var id=_.attr(grandparent, 'for')
			_.attr(field, 'id', _.attr(grandparent, 'for'))
			}
		};
		_.attr(field, 'style', 'display:none');
		// creation des checkboxs 
		var nboptions = $(select).find('option').length;
		for (var index=0; index<nboptions; index++) {
			var pm = _.create('span')
			_.addClass(pm, 'pk-checkbox pk-checkbox-unselected -'+ config.theme);
			_.attr(pm, 'value', select[index].getAttribute("value"));
			_.attr(pm, 'style', select[index].getAttribute("style"));
			pm.innerHTML = '<span class="pk-checkbox-button"></span>' +select[index].innerHTML;
			parent.appendChild(pm);
		}
		parent.appendChild(field);
		// Event listener
		_.events(parent, 'click', function(e){
			e.preventDefault();
			e.stopPropagation();
		if(parent.hasAttribute('for')) {
			var label=parent;
		}
		else {
			var label=grandparent;
			}
		if(!$(event.target).closest('.pk-slide').length) {
			if ($(event.target).hasClass('pk-checkbox')) setSelectedValue((event.target));
			if ($(event.target).hasClass('pk-checkbox-button')) setSelectedValue((event.target).parentElement);
		} 
		});

	}

	function feedField(select, value){
		select.parentElement.querySelector('.pk-slide').innerHTML = value;
	}

	/**
	 * Sets a value (option) default for field select
	 */
	function setInitialValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			feedField(select, select[select.selectedIndex].innerHTML);
		});
	}
		/**
	 * Sets a next value (option) for field select
	 */
	function setSelectedValue(select){
		parent = $(select).parent();
		var selectblur = (parent).children('select');
		var valuesearch = _.attr(select, "Value");
		var currentselect = select;
		parent = $(select).parent();
		select = (parent).children('select');
		var nboptions = $(select[0]).find('option').length;
		for (var index=0; index<nboptions; index++) {
			var value = _.attr((select[0][index]), 'value');
			if (value  == "X") {
				if ('selected' != _.attr((select[0][index]), 'selected')) { // non selected  en selected
					_.attr((select[0][index]), 'selected', 'selected');
				}
				else { //  selected  en non selected
					(select[0][index]).removeAttribute('selected');
				}
				
			}
			else if (valuesearch == value) {
				if ('selected' != _.attr((select[0][index]), 'selected')) { // non selected  en selected
					_.attr((select[0][index]), 'selected', 'selected');
					_.rmClass((currentselect), 'pk-checkbox-unselected');
					_.addClass((currentselect), 'pk-checkbox-selected');
					var texte = (select[0][index]).innerHTML; 
					$(parent).children('.inputcheckbox').html(texte);	
				}
				else { //  selected  en non selected
					(select[0][index]).removeAttribute('selected');
					_.rmClass((currentselect), 'pk-checkbox-selected');
					_.addClass((currentselect), 'pk-checkbox-unselected');
					$(parent).children('.inputcheckbox').html("");
				}
			}
		}
		$(selectblur).blur();


	}

	/**
	 * Sets a value (option) vide for field select : JPD
	 */
	function setEmptyValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			var placeholder = _.attr(select, 'placeholder');
			$("option:selected").removeAttr("selected");
			parent = $(select).parent();
			var oldselect = (parent).children('span.pk-checkbox-selected'); 
			if (oldselect.length) {
				_.rmClass((oldselect[0]), 'pk-checkbox-selected');
				_.addClass((oldselect[0]), 'pk-checkbox-unselected');
			}
		});
	}	

	/**
	 * Merges the settings passed by the user with the default settings of the package, adding their own configurations
	 */
	function mergeToDefaults(config){
		
		ownConfig = JSON.parse(JSON.stringify(defaults));

		for (var item in config) {
			if (config.hasOwnProperty(item)) {
				ownConfig[item] = config[item];
			}
		}

	}	

	// Revealing the methods that shall be public
	return {
		to : init,
		updated : setInitialValue,
		empty : setEmptyValue
		};


})();

if (typeof module !== 'undefined' && module.exports) {
    module.exports = checkboxselect;
}
