/*
 * MultiDatesPicker v1.6.4
 * http://multidatespickr.sourceforge.net/
 * 
 * Copyright 2014, Luca Lauretta
 * Dual licensed under the MIT or GPL version 2 licenses.
 */
var plxdatmulti = (function( $ ){
	$.extend($.ui, { multiDatesPicker: { version: "1.6.4" } });
	
	"use strict";

//	console.time('BMPickout');	

	// Own configuration of each field select
	var ownConfig = {};

	// Default values
	var defaults = {
		theme : 'clean',
		addDates : "",
		resetDates : "",
		noResults : 'No Results',
		multiple : true,
		txtBtnMultiple : 'CONFIRM SELECTED'
	};

	/**
	 * Utilities
	 * Included in version 1.2.0
	 */
	var _ = {
		create : function (tag){
			return document.createElement(tag);
		},
		attr : function(el, attr, value){
			if (!value) {
				return el.getAttribute(attr);
			}

			if (Array.isArray(el)) {
				el.forEach(function(element) {
					element.setAttribute(attr, value);					
				});
				return;
			}

			el.setAttribute(attr, value);
		},
		events : function(el, type, callback) {
			el.addEventListener(type, callback, false);
		},
		toArray : function(el){
			return [].slice.call(el);
		},
		addClass : function(el, selector){
			var self = this;
			var cls = !!self.attr(el, 'class') ? self.attr(el, 'class') : '';
			self.attr(el, 'class',  cls+ ' '+selector);
		},
		addId : function(el, selector){
			var self = this;
			self.attr(el, 'id',  selector);
		},
		rmClass : function(el, selector){
			var self = this;
			if (Array.isArray(el)) {
				el.forEach(function(element) {
					self.attr(element, 'class', self.attr(element, 'class').replace(' '+selector, ''));					
				});
				return;
			}
			self.attr(el, 'class', self.attr(el, 'class').replace(' '+selector, ''));
		},
		$ : function(selector, el) {
			return (el || document).querySelector(selector);
		},
		$$ : function(selector, el) {
			return this.toArray((el || document).querySelectorAll(selector));
		}
	};
	
	function init(config){
		setElement(config);
		prepareElement();
	}
	
	/**
	 * Defines the own configuration and assigns the select
	 * @param {[type]} config = String or object to setting
	 */
	function setElement(config){

		var objConfig = typeof config === 'object' ? config : {};

		if (typeof config === 'string') {
			objConfig.el = config;
		}

		// Retrieve the DOM to be manipulated
		objConfig.DOM = _.$$(objConfig.el);

		mergeToDefaults(objConfig);		
	}
	
	/**
	 * Prepare the elements that will be handled by the module
	 */
	function prepareElement(){

		ownConfig.DOM.map(function(select, index){
			createElements(select, index);
		});
	}

	function createElements(select, index){

		// Cache self config 
		var config = ownConfig,
			isMultiple = true;
		$(select).removeClass("plxdatmulti");
//		select.style.display = 'none';
		var parent = select.parentElement;
		var grandparent = select.parentElement.parentElement;
		_.attr(parent, 'style', 'position:relative;');
		var placeholder = _.attr(select, 'placeholder');
		// Visual element simule field input
		var field = _.create('div');
		_.addClass(field, 'plxdatmulti -'+ config.theme + (!!isMultiple ? ' -multiple' : ''));
		if(!!placeholder) field.innerHTML = placeholder;
		if(parent.hasAttribute('for')) {
			_.addClass(field, _.attr(parent, 'for'));
			iddatepicker = _.attr(parent, 'for');
		}
		else {
			if(grandparent.hasAttribute('for')) _.addClass(field, _.attr(grandparent, 'for'));
			iddatepicker = _.attr(grandparent, 'for');
		}
		_.addId(select, iddatepicker); // pose d'un ID sur l'input = label for 
		_.addId(field, iddatepicker+"-calendar"); // pose d'un ID sur la div  = label for + "-calendar"
		parent.appendChild(field);
		var field = _.create('div');
		_.addClass(field, "button-calendar");
		field.innerText = "Effacer tout";
		parent.appendChild(field);
		var datas_el = ownConfig.addDates.filter((check) => check[0] ==  iddatepicker)
		search = iddatepicker;
		datas = ownConfig.addDates; 
		var datas_el = datas.filter((check) => check[0] ==  search)
		var newvalue = "";
		for (labelarray of datas_el) {
			search = labelarray[0];
			newvalue = labelarray[1];
		}
		if (newvalue == "") newvalue = []
		else newvalue = newvalue.split(',');
		$('.'+iddatepicker).multiDatesPicker (
			{
			addDates: newvalue,			
			dateFormat: "m-d-yy",
			separator: ",",
			altField: "#"+iddatepicker
			}
		)
				// Event listener
		_.events(parent, 'click', function(e){
			e.preventDefault();
			e.stopPropagation();
			if(parent.hasAttribute('for')) {
				var label=parent;
			}
			else {
				var label=grandparent;
				}
			if(!$(event.target).closest('.plxdatmulti').length) {
				if ($(event.target).hasClass('button-calendar')) setEmptyValue(event);
			}
		});
	}
	
	function feedField(select, value){
		select.parentElement.querySelector('.plxdatmulti').innerHTML = value;
//		document.getElementById("circuit").value = value; // special pour le site loriginal (bug sinon)
	}
	/**
	 * RAZ des dates et remises des dates passées en option resetDates 
	 */
	function setInitialValue(config){
		setElement(config);	
		var initdates = ownConfig.resetDates;
		var mindate = ownConfig.minDate;
		var maxdate = ownConfig.maxDate;

		initdates = initdates.replace(/ /g, "");
		ownConfig.DOM.forEach(function(select){
			$(select).multiDatesPicker('resetDates', 'picked', 'minDate');
			$(select).multiDatesPicker (
				{
				addDates: initdates.split(','),	
				minDate : mindate,
				maxDate : maxdate
				}
			)
		});
	}
	/**
	 * RAZ de toutes les dates 
	 */
	function setEmptyValue(select){
		parent = $(select.target).parent();
		select = (parent).children('.plxdatmulti');
		$(select).multiDatesPicker('resetDates', 'picked');
	}	
	/**
	 * Merges the settings passed by the user with the default settings of the package, adding their own configurations
	 */
	function mergeToDefaults(config){
		
		ownConfig = JSON.parse(JSON.stringify(defaults));

		for (var item in config) {
			if (config.hasOwnProperty(item)) {
				ownConfig[item] = config[item];
			}
		}

	}
	// Revealing the methods that shall be public

	return {
		to : init,
		updated : setInitialValue,
		empty : setEmptyValue 
	};

})( jQuery );
if (typeof module !== 'undefined' && module.exports) {
    module.exports = plxdatmulti;
}