/*
*	plxtoggle - Cool effect for field select on form
*/

var plxtoggle = (function(){

	"use strict";	

	// Own configuration of each field select
	var ownConfig = {};
	// Default values
	var defaults = {
		theme : 'clean',
		toggle_open : false,
		noResults : 'No Results'
	};

	/**
	 * Utilities
	 * Included in version 1.2.0
	 */
	var _ = {
		create : function (tag){
			return document.createElement(tag);
		},
		attr : function(el, attr, value){
			if (!value) {
				return el.getAttribute(attr);
			}

			if (Array.isArray(el)) {
				el.forEach(function(element) {
					element.setAttribute(attr, value);					
				});
				return;
			}

			el.setAttribute(attr, value);
		},
		events : function(el, type, callback) {
			el.addEventListener(type, callback, false);
		},
		toArray : function(el){
			return [].slice.call(el);
		},
		addClass : function(el, selector){
			var self = this;
			var cls = !!self.attr(el, 'class') ? self.attr(el, 'class') : '';
			self.attr(el, 'class',  cls+ ' '+selector);
		},
		rmClass : function(el, selector){
			var self = this;
			if (Array.isArray(el)) {
				el.forEach(function(element) {
					self.attr(element, 'class', self.attr(element, 'class').replace(' '+selector, ''));					
				});
				return;
			}
			self.attr(el, 'class', self.attr(el, 'class').replace(' '+selector, ''));
		},
		$ : function(selector, el) {
			return (el || document).querySelector(selector);
		},
		$$ : function(selector, el) {
			return this.toArray((el || document).querySelectorAll(selector));
		}
	};

	/**
	 * Starts the module preparing the elements
	 * @param config = String or object to setting
	 */
	function init(config){
		setElement(config);
		prepareElement();
	}

	/**
	 * Defines the own configuration and assigns the select
	 * @param {[type]} config = String or object to setting
	 */
	function setElement(config){

		var objConfig = typeof config === 'object' ? config : {};

		if (typeof config === 'string') {
			objConfig.el = config;
		}

		// Retrieve the DOM to be manipulated
		objConfig.DOM = _.$$(objConfig.el);

		mergeToDefaults(objConfig);		
	}


	/**
	 * Prepare the elements that will be handled by the module
	 */
	function prepareElement(){

		ownConfig.DOM.map(function(select, index){
			createElements(select, index);
		});
	}

	function createElements(select, index){

		// Cache self config 
		var config = ownConfig;

//		select.style.display = 'none';

		var parent = select.parentElement;
		var grandparent = select.parentElement.parentElement;
//		_.attr(parent, 'style', 'position:relative;float:left;');
//		_.attr(parent, 'style', '');
		var placeholder = _.attr(select, 'placeholder');

		// Visual element simule field input
		var field = _.create('div');
		_.addClass(field, 'pk-slide -'+ config.theme);
		if(!!placeholder) field.innerHTML = placeholder;
		if(parent.hasAttribute('for')) {
			var id=_.attr(parent, 'for')
			_.attr(field, 'id', _.attr(parent, 'for'))
		}
		else {
			if(grandparent.hasAttribute('for')) {
			var id=_.attr(grandparent, 'for')
			_.attr(field, 'id', _.attr(grandparent, 'for'))
			}
		};
		var pm = _.create('span');
		_.addClass(pm, 'toggle-p -');			
		select.before(pm);
		_.addClass(select, ' -'+ config.theme);
		let element = $(parent).children().first();
		if ($(element[0]).hasClass('toggle-m') && !config.toggle_open) {
			setInitInvisible(element[0], parent)
		}
	
		_.events(parent, 'click', function(e){
			e.preventDefault();
			e.stopPropagation();
			setVisibleInvisible(event, parent)		
		});
	}

	/**
	 * Event for select field simple
	 * @param {HTMLObject} select
	 * @param {Object} data
	 */
	function setOptionSimple(select, data, txt){
		_.toArray(select).map(function(option, index){
			if (index === data.index) {
				_.attr(option, 'selected', 'true');
//				_.attr(option, 'selected', 'selected');
				return;
			}

			option.removeAttribute('selected');
		});
		feedField(select, data.txt);		
		$(select).blur(); // ajout JPD : permet de forcer le contrôle sur le champ 

	}

	function feedField(select, value){
		select.parentElement.querySelector('.pk-slide').innerHTML = value;
//		document.getElementById("circuit").value = value; // special pour le site loriginal (bug sinon)
	}

	/**
	 * Sets a value (option) default for field select
	 */
	function setInitialValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			feedField(select, select[select.selectedIndex].innerHTML);
		});
	}
	/**
	 * Sets init 
	 */
	function setInitInvisible(select, element){
		parent = $(select).parent();
		select = (parent).next();
		if((select).hasClass('plxToggle_zone_visible')) {
		$(select).slideToggle(200);
		$(select).removeClass("plxToggle_zone_visible");
		$(select).addClass("plxToggle_zone_invisible");
		select = (parent).children().first();
		$(select).removeClass("toggle-m");
		$(select).addClass("toggle-p");
		}
	}
			/**
	 * Sets a prev value (option) for field select
	 */
	function setVisibleInvisible(select, element){
		parent = $(select.target).parent();
		select = (parent).next();
		if((select).hasClass('plxToggle_zone_visible')) {
		$(select).slideToggle(200);
		$(select).removeClass("plxToggle_zone_visible");
		$(select).addClass("plxToggle_zone_invisible");
		select = (parent).children().first();
		$(select).removeClass("toggle-m");
		$(select).addClass("toggle-p");
		}
		else {
			if((select).hasClass('plxToggle_zone_invisible')) {
			$(select).slideToggle(200);
			$(select).removeClass("plxToggle_zone_invisible");
			$(select).addClass("plxToggle_zone_visible");
			select = (parent).children().first();
			$(select).removeClass("toggle-p");
			$(select).addClass("toggle-m");
			}
		}
	}
	/**
	 * Sets a value (option) vide for field select : JPD
	 */
	function setEmptyValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			var placeholder = _.attr(select, 'placeholder');
			$("option:selected").removeAttr("selected");
			feedField(select, select[0].innerHTML);
		});
	}	

	/**
	 * Merges the settings passed by the user with the default settings of the package, adding their own configurations
	 */
	function mergeToDefaults(config){
		
		ownConfig = JSON.parse(JSON.stringify(defaults));

		for (var item in config) {
			if (config.hasOwnProperty(item)) {
				ownConfig[item] = config[item];
			}
		}

	}	

	// Revealing the methods that shall be public
	return {
		to : init,
		updated : setInitialValue,
		empty : setEmptyValue
		};


})();

if (typeof module !== 'undefined' && module.exports) {
    module.exports = plxtoggle;
}
