/*
*	slidepm - Cool effect for field select on form
*	Copyright (c) 2016 Ktquez
*	Project repository: https://github.com/ktquez/slidepm
* 	MIT license.
*/

var slidepm = (function(){

	"use strict";	

	// Own configuration of each field select
	var ownConfig = {};
	// Default values
	var defaults = {
		theme : 'clean',
		search : false,
		noResults : 'No Results'
	};

	/**
	 * Utilities
	 * Included in version 1.2.0
	 */
	var _ = {
		create : function (tag){
			return document.createElement(tag);
		},
		attr : function(el, attr, value){
						if (typeof(value) == 'undefined') {
							return el.getAttribute(attr);
						}
						if (Array.isArray(el)) {
							el.forEach(function(element) {
							element.setAttribute(attr, value);	
						});
						return;
						}						
						el.setAttribute(attr, value);
		},
		events : function(el, type, callback) {
			el.addEventListener(type, callback, false);
		},
		toArray : function(el){
			return [].slice.call(el);
		},
		addClass : function(el, selector){
			var self = this;
			var cls = !!self.attr(el, 'class') ? self.attr(el, 'class') : '';
			self.attr(el, 'class',  cls+ ' '+selector);
		},
		rmClass : function(el, selector){
			var self = this;
			if (Array.isArray(el)) {
				el.forEach(function(element) {
					self.attr(element, 'class', self.attr(element, 'class').replace(' '+selector, ''));					
				});
				return;
			}
			self.attr(el, 'class', self.attr(el, 'class').replace(' '+selector, ''));
		},
		$ : function(selector, el) {
			return (el || document).querySelector(selector);
		},
		$$ : function(selector, el) {
			return this.toArray((el || document).querySelectorAll(selector));
		}
	};

	/**
	 * Starts the module preparing the elements
	 * @param config = String or object to setting
	 */
	function init(config){
		setElement(config);
		prepareElement();
	}

	/**
	 * Defines the own configuration and assigns the select
	 * @param {[type]} config = String or object to setting
	 */
	function setElement(config){

		var objConfig = typeof config === 'object' ? config : {};

		if (typeof config === 'string') {
			objConfig.el = config;
		}

		// Retrieve the DOM to be manipulated
		objConfig.DOM = _.$$(objConfig.el);

		mergeToDefaults(objConfig);		
	}


	/**
	 * Prepare the elements that will be handled by the module
	 */
	function prepareElement(){

		ownConfig.DOM.map(function(select, index){
			createElements(select, index);
		});
	}

	function createElements(select, index){

		// Cache self config 
		var config = ownConfig;

		select.style.display = 'none';

		var parent = select.parentElement;
		var grandparent = select.parentElement.parentElement;
		var great_grandparent = select.parentElement.parentElement.parentElement;
//		_.attr(parent, 'style', 'position:relative;float:left;');
//		_.attr(parent, 'style', '');
		var placeholder = _.attr(select, 'placeholder');

		// Visual element simule field input
		var field = _.create('div');
		_.addClass(field, 'pk-slide -'+ config.theme);
		if(!!placeholder) field.innerHTML = placeholder;
		if(parent.hasAttribute('for')) {
			_.attr(field, 'id', _.attr(parent, 'for'))
		}
		else {
			if(grandparent.hasAttribute('for')) {
			_.attr(field, 'id', _.attr(grandparent, 'for'))
			}
			else {
			if(great_grandparent.hasAttribute('for')) _.attr(field, 'id', _.attr(great_grandparent, 'for'));
			}
		};
		var pm = _.create('span');
		_.addClass(pm, 'pk-m');	
		parent.appendChild(pm);
		parent.appendChild(field);
		var pm = _.create('span');
		_.addClass(pm, 'pk-pm pk-p');	
		parent.appendChild(pm);

		// Event listener
		_.events(parent, 'click', function(e){
			e.preventDefault();
			e.stopPropagation();
		if(parent.hasAttribute('for')) {
			var label=parent;
		}
		else {
			var label=grandparent;
			}
		if(!$(event.target).closest('.pk-slide').length) {
			if ($(event.target).hasClass('pk-m')) setPrevValue(event, parent);
			if ($(event.target).hasClass('pk-p')) setNextValue(event, parent);
		} 
		});

	}

	/**
	 * Create by options group 
	 * @param  {Object} optGroup
	 * @param  {object} main
	 * @param  {object} config 
	 */
	function createOptGroup(optGroup, main, config) {

		var titleGroup = _.create('li');
		_.addClass(titleGroup, 'pk-option-group -'+config.theme);
		_.attr(titleGroup, 'data-opt-group', optGroup.label);
		_.attr(titleGroup, 'data-type', optGroup.localName);
		titleGroup.innerHTML = optGroup.label.toUpperCase();
		main.appendChild(titleGroup);
		
		return titleGroup;
	}

	function feedField(select, value){
		value = value.replace(" / ", "</br>"); 
		select.parentElement.querySelector('.pk-slide').innerHTML = value;
//		document.getElementById("circuit").value = value; // special pour le site loriginal (bug sinon)
	}

	/**
	 * Sets a value (option) default for field select
	 */
	function setInitialValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			feedField(select, select[select.selectedIndex].innerHTML);
		});
	}
		/**
	 * Sets a next value (option) for field select
	 */
	function setNextValue(select, element){
		parent = $(select.target).parent();
		select = (parent).children('select');
		var nboptions = $(select[0]).find('option').length;
		var oldindex = $(select).prop('selectedIndex');
		if(typeof(oldindex)  === "undefined") {oldindex = -1};
		(select[0][oldindex]).removeAttribute('selected');
		var i = oldindex + 1;
		var newindex = oldindex;
		while (i <= nboptions - 1) {
			if ($(select[0][i]).css('display') != 'none') {
				var newindex = i;
				break;
			}
			else {
				i++;
			};
		}
			feedField(element, (select[0][newindex]).innerHTML);
//			_.attr((select[0][newindex]), 'selected', 'true');
			_.attr((select[0][newindex]), 'selected', 'selected');
			$(select).blur(); // ajout JPD : permet de forcer le contrôle sur le champ 
	}
			/**
	 * Sets a prev value (option) for field select
	 */
	function setPrevValue(select, element){
		parent = $(select.target).parent();
		select = (parent).children('select');
		var nboptions = $(select[0]).find('option').length;
		var oldindex = $(select).prop('selectedIndex');
		if(typeof(oldindex)  === "undefined") {oldindex = -1};
		(select[0][oldindex]).removeAttribute('selected');
		var i = oldindex - 1;
		var newindex = oldindex;
		while (i >= 0) {
			if ($(select[0][i]).css('display') != 'none') {
				var newindex = i;
				break;
			}
			else {
				i--;
			};
		}
			feedField(element, (select[0][newindex]).innerHTML);
//			_.attr((select[0][newindex]), 'selected', 'true');
			_.attr((select[0][newindex]), 'selected', 'selected');
			$(select).blur(); // ajout JPD : permet de forcer le contrôle sur le champ 

	}
	/**
	 * Sets a value (option) vide for field select : JPD
	 */
	function setEmptyValue(config){
		setElement(config);	
		ownConfig.DOM.forEach(function(select){
			var placeholder = _.attr(select, 'placeholder');
			$("option:selected").removeAttr("selected");
			feedField(select, select[0].innerHTML);
//			feedField(select, placeholder);
//			$(select).value("");
		});
	}	

	/**
	 * Merges the settings passed by the user with the default settings of the package, adding their own configurations
	 */
	function mergeToDefaults(config){
		
		ownConfig = JSON.parse(JSON.stringify(defaults));

		for (var item in config) {
			if (config.hasOwnProperty(item)) {
				ownConfig[item] = config[item];
			}
		}

	}	

	// Revealing the methods that shall be public
	return {
		to : init,
		updated : setInitialValue,
		empty : setEmptyValue
		};


})();

if (typeof module !== 'undefined' && module.exports) {
    module.exports = slidepm;
}
